#!/usr/bin/env Rscript

# Load optparse we need to check inputs
suppressPackageStartupMessages(require(optparse))
# Load common functions
suppressPackageStartupMessages(require(workflowscriptscommon))

# create a CDS object from raw expression matrix. 
# The CDS will then be used as input in further steps of the workflow

# parse options 
option_list = list(
    make_option(
        c("-e", "--expression-matrix"),
        action = "store",
        default = NA,
        type = 'character',
        help = "Numeric matrix of expression values; rows: genes, columns: cells.
        See http://cole-trapnell-lab.github.io/monocle-release/docs/#getting-started-with-monocle
        for explanation"
    ), 
    make_option(
        c("-p", "--phenotype-data"),
        action = "store",
        default = NA,
        type = 'character',
        help = "Table of phenotype data; rows: cells, columns: cell attributes 
                (such as cell type, culture condition, day captured, etc.)"
    ),
    make_option(
        c("-f", "--feature-data"),
        action = "store",
        default = NA,
        type = 'character',
        help = "Table of gene features, rows:features (e.g. genes), columns:
                gene attributes (such as biotype, gc content, etc.)"
    ),
    make_option(
        c("-o", "--output-file"),
        action = "store",
        default = NA,
        type = 'character',
        help = "output file for CDS object in .rds format"
    )
)

opt = wsc_parse_args(option_list, mandatory = c('expression_matrix', 
                                                'phenotype_data',
                                                'feature_data', 
                                                'output_file'))

# check parameters are correctly defined 
if(! file.exists(opt$expression_matrix)){
    stop((paste('File ', opt$expression_matrix, 'does not exist')))
}

if(! file.exists(opt$phenotype_data)){
    stop((paste('File ', opt$phenotype_data, 'does not exist')))
}

if(! file.exists(opt$feature_data)){
    stop((paste('File ', opt$feature_data, 'does not exist')))
}


# if input is OK, load the package
suppressPackageStartupMessages(require(garnett))

# initialise the CDS object 
expr_matrix = Matrix::readMM(opt$expression_matrix)
pData = read.table(opt$phenotype_data, sep="\t")
fData = read.table(opt$feature_data)
row.names(expr_matrix) = row.names(fData)
colnames(expr_matrix) = row.names(pData)

cds = new_cell_data_set(as(expr_matrix, "dgCMatrix"), cell_metadata = pData, gene_metadata = fData)
saveRDS(cds, file = opt$output_file)